<?php

namespace App\Http\Controllers;

use App\Http\Requests\UserCardRequest;
use App\Models\User;
use App\Models\UserCard;
use App\Traits\HashId;
use App\Traits\Response;
use Exception;
use Illuminate\Http\Request;
use Stripe\StripeClient;

class UserCardController extends Controller
{
    use HashId, Response;

    protected $return_url;
    protected $stripe;

    public function __construct()
    {

        $this->return_url = url('/card/callback');
        $this->stripe = new StripeClient(env('STRIPE_SECRET'));
    }


    public function create(UserCardRequest $request)
    {

        try {

            $user = auth()->user();
            if (!$user->customer_stripe_id) {
                $user->customer_stripe_id = $this->createStripeCustomer($user);
            }

            $card = $this->stripe->customers->createSource($user->customer_stripe_id, ['source' => $request->token]);
            $source = $this->stripe->setupIntents->create([
                'payment_method_types' => ['card'],
                'customer' => $user->customer_stripe_id,
                'usage' => 'off_session',
                'confirm' => true,
                'payment_method' => $card->id,
                'return_url' => url("/card/callback/" . $this->encode($user->id)),
                'metadata' => [
                    'card_id' => $card->id,
                    'last_4' => $card->last4,
                    'brand' => $card->brand,
                    'is_default' => $request->is_default
                ]
            ]);

            $user_card = [];
            if ($source->status == 'succeeded') {

                $user_card = $user->card()->create([
                    'card_id' => $source->status == 'succeeded' ? $card->id : $source->id,
                    'last_4' => $card->last4,
                    'brand' => $card->brand,
                    'is_default' => $request->is_default,
                    'status' => $source->status
                ]);
            } elseif ($source->next_action->type) {
                $user_card['status'] = $source->status;
                $user_card['url'] = $source->next_action->redirect_to_url->url;
            } else {
                \Log::debug('CARD ERROR');
                return $this->response(null, 422, false, ['Something wents wrong while creating card']);
            }

            return $this->response(['card' => $user_card, 402, false, ['Payment action required']]);
        } catch (Exception $e) {
            \Log::info($e);
            // return $this->response(null, 422, false, ['Invalid card details!']);
            return $this->response(null, 422, false, [$e->getMessage()]);
        }
    }

    public function showAll()
    {
        return $this->response(["cards" => auth()->user()->card], 200);
    }

    public function delete(Request $request)
    {

        try {

            $customer = auth()->user();

            $this->stripe->customers->deleteSource(
                $customer->customer_stripe_id,
                $request->card_id,
                []
            );

            UserCard::where('card_id', $request->card_id)->delete();

            return $this->response(null, 200, false, ['Card deleted successfully']);
        } catch (Exception $e) {
            return $this->response(null, 422, false, [$e->getMessage()]);
        }
    }

    public function callback(Request $request, $id)
    {

        $user = User::findOrFail($this->decode($id));
        $intent = $this->stripe->setupIntents->retrieve($request->setup_intent, []);

        if ($intent->status == 'succeeded') {
            $card = $intent->metadata;
            $user_card = $user->card()->create([
                'card_id' => $card->card_id,
                'last_4' => $card->last_4,
                'brand' => $card->brand,
                'is_default' => $card->is_default,
                'status' => $intent->status
            ]);
        }

        return view('confirm-payment', compact('intent'));
    }


    public function createStripeCustomer($customer)
    {

        $customer_stripe = $this->stripe->customers->create([
            'name' => $customer['full_name'],
            'email' => $customer['email'],
            'phone' => $customer['phone'],
            'payment_method' => 'pm_card_visa',
        ]);

        $customer->update([
            'customer_stripe_id' => $customer_stripe['id']
        ]);

        return $customer_stripe['id'];
    }

    public function createCardToken(Request $request)
    {
        $card = $this->stripe->tokens->create([
            'card' => [
                'number' => $request->number,
                'exp_month' => $request->exp_month,
                'exp_year' => $request->exp_year,
                'cvc' => $request->cvc,
            ],
        ]);

        return $this->response(["card" => $card], 200, false, ['Card created successfully']);
    }
}
