<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;

use App\Helpers\Helper;
use App\Traits\RandomValue;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\DB;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Support\Str;
use Symfony\Component\HttpKernel\Profiler\Profile;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, RandomValue;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [

        'full_name',
        'email',
        'social_id',
        'provider',
        'phone',
        'password',
        'gender',
        'age',
        'service_charge',
        'profile_image',
        'customer_stripe_id',
        'user_type',
        'user_role',
        'ethnicity',
        'language',
        'biography',
        'pet_details',
        'family_details',
        'qualification',
        'referral_code',
        'referred_by',
        'longitude',
        'latitude',
        'is_approved',
        'is_online',
        'service_charge',
        'experiences',
        'verification_status',
        'candidate_id',
        'dob',
        'ssn',
        'zipcode',
        'work_locations'
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    /**
     * appends
     *
     * @var array
     */
    protected $appends = [
        'rating',
        'rating_count',
        'rating_list',
        'is_blocked',
        'is_favorite',
        'is_subscribed',
    ];

    /**
     * The booted method of the model
     */
    protected static function booted()
    {
        static::creating(function ($user) {
            $user->verification_code = mt_rand(10000, 99999);
            $user->phone_verification_code = mt_rand(10000, 99999);
            $user->expired_at = Carbon::now()->addMinutes(60);
        });
    }


    /**
     * verifyUser
     *
     * @return void
     */
    public function verifyUser()
    {
        $this->phone_verified_at = now();
        $this->phone_verification_code = NULL;
        $this->save();
    }

    /**
     * Set password encryption when insert
     *
     * @param $password
     */
    public function setPasswordAttribute($password)
    {
        $this->attributes['password'] = bcrypt($password);
    }

    /**
     * getProfileImageAttribute
     *
     * @return void
     */
    public function getProfileImageAttribute()
    {
        return $this->attributes['profile_image'] !== null ? env('MEDIA_URL') . $this->attributes['profile_image'] : null;
    }

    /**
     * Get formatted timestamp of email_verified_at
     *
     * @param $date
     * @return string
     */
    public function getExpiredAtAttribute($date)
    {
        if ($date !== null) {
            return Carbon::parse($date)->format('Y-m-d H:i:s');
        }
    }

    /**
     * getRatingAttribute
     *
     * @return void
     */
    public function getRatingAttribute()
    {
        return $this->ratings()->avg('rating');
    }

    /**
     * getRatingCountAttribute
     *
     * @return void
     */
    public function getRatingCountAttribute()
    {
        $count = $this->ratings()->select('appointment_id', \DB::raw('COUNT(*) as rating_count'))
    ->groupBy('appointment_id')
    ->get();
    return count($count);
    }

    /**
     * getRatingListAttribute
     *
     * @return void
     */
    public function getRatingListAttribute()
    {
        return $this->ratings()
            ->selectRaw('type, AVG(rating) as average_rating')
            ->groupBy('type')
            ->get();
    }

    /**
     * getIsBlockedAttribute
     *
     * @return void
     */
    public function getIsBlockedAttribute()
    {
        $authenticatedUserId = auth()->id();

        return $this->belongsToMany(User::class, 'block_users', 'blocked_id', 'user_id')
            ->where('user_id', $authenticatedUserId)
            ->exists();
    }

    /**
     * getIsFavoriteAttribute
     *
     * @return void
     */
    public function getIsFavoriteAttribute()
    {
        $authenticatedUserId = auth()->id();

        return $this->belongsToMany(User::class, 'favorite_users', 'favorite_id', 'user_id')
            ->where('user_id', $authenticatedUserId)
            ->exists();
    }

    public function getIsSubscribedAttribute()
    {
        $subscription = $this->subscriptions()->latest()->first();
        if($subscription){
            if($subscription->end_date >= Carbon::now()->format('d-m-Y')){
                return true;
            }
        }
        return false;
    }

    /**
     * ratings
     *
     * @return void
     */
    public function ratings()
    {
        return $this->hasMany(Rating::class);
    }

    /**
     * Mark user verified
     */
    public function markUserVerified()
    {
        $this->verification_code = null;
        $this->expired_at = null;
        $this->email_verified_at = now();

        $this->save();
    }

    /**
     * Check if verification code is not expired
     *
     * @return bool
     */
    public function hasValidCode()
    {
        return $this->verification_code !== null && Carbon::parse($this->expired_at)->addMinutes(60) > now();
    }

    /**
     * card
     *
     * @return void
     */
    public function card()
    {
        return $this->hasMany(UserCard::class);
    }

    /**
     * devices
     *
     * @return void
     */
    public function devices()
    {
        return $this->hasMany(UserDevice::class);
    }

    /**
     * helpAndSupport
     *
     * @return void
     */
    public function helpAndSupport()
    {
        return $this->hasMany(HelpAndSupport::class);
    }

    /**
     * userMedia
     *
     * @return void
     */
    public function userMedia()
    {
        return $this->hasMany(UserMedia::class);
    }

    /**
     * userPreference
     *
     * @return void
     */
    public function userPreference()
    {
        return $this->hasOne(UserPreferences::class);
    }

    /**
     * userDevice
     *
     * @param  mixed $request
     * @param  mixed $access_token
     * @return void
     */
    public function userDevice($request, $access_token)
    {
        $request['access_token'] = $access_token;
        $this->devices()->updateOrCreate(
            $request->only('udid'),
            $request->all()
        );
    }

    /**
     * subscriptions
     *
     * @return void
     */
    public function subscriptions()
    {
        return $this->hasMany(Subscription::class);
    }

    /**
     * preferences
     *
     * @return void
     */
    public function preferences()
    {
        return $this->belongsToMany(Preference::class, 'user_preferences', 'user_id');
    }

    /**
     * categories
     *
     * @return void
     */
    public function categories()
    {
        return $this->belongsToMany(Category::class, 'user_categories', 'user_id');
    }

    /**
     * wallet
     *
     * @return void
     */
    public function wallet()
    {
        return $this->hasOne(Wallet::class);
    }

    /**
     * scopeExcludeBlocked
     *
     * @param  mixed $query
     * @return void
     */
    public function scopeExcludeBlocked($query)
    {
        $blockedUsers = auth()->user()->blockedUsers()->pluck('blocked_id');

        return $query->whereNotIn('users.id', $blockedUsers);
    }

    /**
     * scopeNeedAsap
     *
     * @param  mixed $query
     * @return void
     */
    public function scopeNeedAsap($query)
    {
        return $query->whereRaw(" ( SELECT COUNT(id) FROM appointments as a WHERE service_provider_id = users.id AND date = DATE( NOW() ) ) = 0 ");
    }

    /**
     * scopeHasCategory
     *
     * @param  mixed $query
     * @param  mixed $category
     * @return void
     */
    public function scopeHasCategory($query, $category)
    {
        return $query->join('user_categories as ca', 'ca.user_id', '=', 'users.id')
            ->whereIn('category_id', $category);
    }

    /**
     * scopeHasSearch
     *
     * @param  mixed $query
     * @param  mixed $category
     * @return void
     */
    public function scopeHasSearch($query, $category)
    {
        return $query->whereHas('categories', function ($q) use ($category) {
            $q->where('title', 'LIKE', '%' . $category . '%');
        });
        //$query->join('user_categories as se', 'se.user_id', '=', 'users.id')
        //  ->where('se.title', 'LIKE', '%'.$category.'%');
    }

    /**
     * scopeHasPreferences
     *
     * @param  mixed $query
     * @param  mixed $preferences
     * @return void
     */
    public function scopeHasPreferences($query, $preferences)
    {
        return $query->join('user_preferences as p', 'p.user_id', '=', 'users.id')
            ->whereIn('preference_id', $preferences);
    }

    /**
     * scopeHasLanguage
     *
     * @param  mixed $query
     * @param  mixed $language
     * @return void
     */
    public function scopeHasLanguage($query, $language)
    {
        $languages = explode(',', $language);
        //dd($languages);
        return
            $query->where(function ($query) use ($languages) {
                foreach ($languages as $languag) {
                    $query->orWhereRaw("FIND_IN_SET('$languag', language)");
                }
            });
        //->whereIn(DB::raw("FIND_IN_SET(language, '".$language."')"),[0]);
        // ->whereIn('language', $language);
    }

    /**
     * scopeHasQualification
     *
     * @param  mixed $query
     * @param  mixed $qualification
     * @return void
     */
    public function scopeHasQualification($query, $qualification)
    {
        return $query
            ->where('qualification', $qualification);
    }

    /*     public function scopeHasRating($query, $qualification)
    {
        return $query
        //->join('user_categories as spc', 'spc.user_id', '=', 'users.id')
        ->whereIn('rating', $qualification );
    } */

    /**
     * scopeHasAge
     *
     * @param  mixed $query
     * @param  mixed $age
     * @return void
     */
    public function scopeHasAge($query, $age)
    {
        return $query

            ->whereBetween('age', array($age[0], $age[1]));
    }

    /**
     * scopeNearBy
     *
     * @param  mixed $query
     * @param  mixed $latitude
     * @param  mixed $longitude
     * @param  mixed $radius
     * @return void
     */
    public function scopeNearBy($query, $latitude, $longitude, $radius = 10)
    {
        $distance = DB::raw("pl.latitude, pl.longitude, 111.111 *
            DEGREES(ACOS(LEAST(1.0, COS(RADIANS(pl.latitude))
                * COS(RADIANS( $latitude ))
                * COS(RADIANS( pl.longitude - $longitude ))
                + SIN(RADIANS( pl.latitude))
                * SIN(RADIANS( $latitude )), 1.0))) AS distance_in_km");

        return $query->join('preferred_locations as pl', 'pl.user_id', '=', 'users.id')
            ->where('pl.latitude', '!=', null)
            ->select([
                'users.id',
                'users.name',
                'users.email',
                'users.profile_image',
                'users.is_online',
                'users.service_provider_type',
                $distance
            ])
            ->having('distance_in_km', '<', $radius)
            ->groupBy('users.id')
            ->orderBy('distance_in_km', 'asc');
    }

    /**
     * schedule
     *
     * @return void
     */
    public function schedule()
    {
        return $this->hasMany(UserSchedule::class);
    }

    /**
     * blockedUsers
     *
     * @return void
     */
    public function blockedUsers()
    {
        return $this->belongsToMany(User::class, 'block_users', 'user_id', 'blocked_id');
    }

    /**
     * favoriteUsers
     *
     * @return void
     */
    public function favoriteUsers()
    {
        return $this->belongsToMany(User::class, 'favorite_users', 'user_id', 'favorite_id');
    }

    /**
     * acitivities
     *
     * @return void
     */
    public function acitivities()
    {
        return $this->hasMany(Activity::class);
    }
}
